@@ -1,4 +1,4 @@
-#
+###############################################################################
 # Forks::Super::Job - object representing a task to perform in
 #                     a background process
 # See the subpackages for some implementation details
@@ -26,7 +26,7 @@ eval "use Devel::GlobalDestruction";
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(@ALL_JOBS %ALL_JOBS);
-our $VERSION = '0.74';
+our $VERSION = '0.75';
 
 our (@ALL_JOBS, %ALL_JOBS, @ARCHIVED_JOBS, $WIN32_PROC, $WIN32_PROC_PID);
 our $OVERLOAD_ENABLED = 0;
@@ -223,12 +223,17 @@ sub toString {
   	grep { defined $job->{$_} } qw(real_pid style cmd exec sub args dir 
                                        start end reaped status closure child_fh
                                        pgid queue_priority timeout expiration
-				       signal_pid);
+				       signal_pid remote);
     my @output = ();
+    no warnings 'uninitialized';
     foreach my $attr (@to_display) {
 	next if ! defined $job->{$attr};
 	if (ref $job->{$attr} eq 'ARRAY') {
 	    push @output, "$attr=[" . join(q{,},@{$job->{$attr}}) . ']';
+        } elsif (ref($job->{attr}) eq 'HASH') {
+            use Data::Dumper;
+            local $Data::Dumper::Indent=0;
+            push @output, "$attr=" . Dumper($job->{$attr});
 	} else {
 	    push @output, "$attr=" . $job->{$attr};
 	}
@@ -362,7 +367,8 @@ sub _can_launch_dependency_check {
 	my $j = $ALL_JOBS{$dj};
 	if (not defined $j) {
 	    carp 'Forks::Super::Job ',
-		    "start dependency $dj for job $job->{pid} is invalid. Ignoring.\n";
+		 "start dependency $dj for job $job->{pid} is invalid. ",
+                 "Ignoring.\n";
 	    next;
 	}
 	if (!$j->is_started) {
@@ -376,6 +382,55 @@ sub _can_launch_dependency_check {
     return 1;
 }
 
+sub _can_launch_remote {
+    my $job = shift;
+    return 1 if !defined $job->{remote};
+    my @hosts;
+    if (ref($job->{remote}{host}) ne 'ARRAY') {
+        @hosts = $job->{remote}{host};
+    } else {
+        @hosts = @{$job->{remote}{host}};
+    }
+    for (my $i=$#hosts; $i>=1; $i--) {
+        my $j = int(rand($i+1));
+        @hosts[$i,$j] = @hosts[$j,$i];
+    }
+
+    foreach my $host (@hosts) {
+        if ($job->_can_launch_remote_check_host($host)) {
+            $job->{remote}{host} = $host;
+            return 1;
+        }
+    }
+    if ($job->{debug}) {
+        debug("_can_launch_remote: host(s) @hosts are too busy");
+    }
+    return 0;
+}
+
+sub _can_launch_remote_check_host {
+    my ($job, $h) = @_;
+    my $max_proc = 
+        defined $job->{max_proc}           ? $job->{max_proc} :
+        exists $Forks::Super::MAX_PROC{$h} ? $Forks::Super::MAX_PROC{$h} :
+        defined $Forks::Super::MAX_PROC    ? $Forks::Super::MAX_PROC : 0;
+    if ($max_proc < 1) {
+        debug("_can_launch_remote: no restriction on host $h")
+            if $job->{debug};
+        return 1;
+    }
+    my $num_active = count_active_processes_on_host($h);
+    if ($num_active >= $max_proc) {
+        debug('_can_launch_remote(): ',
+              "host $h too busy. ($num_active >= $max_proc)") if $job->{debug};
+    } else {
+        debug('_can_launch_remote(): ',
+              "host $h not busy. ($num_active < $max_proc) ",
+              " launch ok") if $job->{debug};
+        return 1;
+    } 
+}
+
 sub _max_proc {
     my $j = shift;
     $j->{max_proc} ||= $j->{max_fork};
@@ -395,13 +450,9 @@ sub _can_launch {
     no warnings qw(once);
 
     my $job = shift;
-    my $max_proc = defined($job->{max_proc})
-	? $job->{max_proc} : $Forks::Super::MAX_PROC;
-    my $max_load = defined($job->{max_load})
-	? $job->{max_load} : $Forks::Super::MAX_LOAD;
-    my $force = defined($job->{max_load}) && $job->{force};
+#   my $force = defined($job->{max_load}) && $job->{force};
 
-    if ($force) {
+    if ($job->{force}) {
 	debug('_can_launch(): force attr set. launch ok')
 	    if $job->{debug};
 	return 1;
@@ -409,7 +460,20 @@ sub _can_launch {
 
     return 0 if not $job->_can_launch_delayed_start_check;
     return 0 if not $job->_can_launch_dependency_check;
+    if ($job->{remote}) {
+        if ($job->_can_launch_remote) {
+            debug('_can_launch_remote(): system not busy. launch ok.')
+                if $job->{debug};
+            return 1;
+        } else {
+            return 0;
+        }
+    }
 
+    my $max_proc = defined($job->{max_proc})
+	? $job->{max_proc} : $Forks::Super::MAX_PROC;
+    my $max_load = defined($job->{max_load})
+	? $job->{max_load} : $Forks::Super::MAX_LOAD;
     if ($max_proc > 0) {
 	my $num_active = count_active_processes();
 	if ($num_active >= $max_proc) {
@@ -433,7 +497,6 @@ sub _can_launch {
 	    return 0;
 	}
     }
-
     debug('_can_launch(): system not busy. launch ok.')
 	if $job->{debug};
     return 1;
@@ -830,7 +893,11 @@ sub _postlaunch_child {
 
     if ($job->{style} eq 'cmd' || $job->{style} eq 'exec') {
 
-	if (defined($job->{fh_config}{stdin})
+        if ($job->{style} eq 'cmd' && $job->{remote}) {
+
+            $job->_postlaunch_child_to_remote_cmd;
+
+        } elsif (defined($job->{fh_config}{stdin})
 	    && defined($job->{fh_config}{sockets})) {
 
 	    $job->_postlaunch_child_to_proc;
@@ -853,7 +920,7 @@ sub _postlaunch_child {
 
         $job->_postlaunch_to_natural_child;
 
-      }
+    }
     return 0;
 }
 
@@ -921,8 +988,8 @@ sub _postlaunch_child_to_cmd {
 
     } else {
 
-        my $retries = $job->{retries} || 0;
 	my $this_pid = $$;
+        my $retries = $job->{retries} || 0;
 	my $exec_pid = _CORE_fork();
 	while (!defined $exec_pid && $retries-- > 0) {
             warn "Forks::Super::Job::_postlaunch_child_to_cmd: ",
@@ -953,6 +1020,77 @@ sub _postlaunch_child_to_cmd {
     exit $c1 >> 8;
 }
 
+sub _postlaunch_child_to_remote_cmd {
+    my $job = shift;
+    my $remote = $job->{remote};
+    debug("Executing command [ @{$job->{cmd}} ] on "
+          . $remote->{host}) if $job->{debug};
+
+    my $retries = $job->{retries} || 0;
+    my $this_pid = $$;
+
+    if ($remote->{proto} eq 'ssh') {
+        if (!$remote->{password} && CONFIG_module("Net::SSH")) {
+
+            # child_fh set/not set
+            # $remote => Net::SSH settings
+            # remote host job accounting
+            my $sshpid = -1;
+            my $host = $remote->{host};
+            my $user = $remote->{user};
+            my $port = $remote->{port};
+            if ($user) {
+                $host = $user . '@' . $host;
+            }
+            if ($port) {
+                $host .= ":$port";
+            }
+                
+
+            my ($in,$out,$err)=map Forks::Super::Job::Ipc::_gensym(),1..3;
+            $sshpid = Net::SSH::sshopen3(
+                $host, $in, $out, $err, @{$job->{cmd}});
+
+            # $sshpid is always valid? yes
+            $job->set_signal_pid($sshpid);
+            if ($job->{fh_config}{in}) {
+                print {$in} <STDIN>;
+            }
+            close $in;
+            my $z = CORE::waitpid $sshpid, 0;
+            my $c1 = $?;
+            print STDOUT <$out>;
+            print STDERR <$err>;
+            deinit_child();
+            debug("waitpid returned $z, exit code of $$ was $c1 ", $c1>>8)
+                if $job->{debug};
+            exit $c1 >> 8;
+        } elsif (CONFIG_module("Net::SSH::Perl")) {
+
+            open TTY,'>/dev/tty';
+
+            my ($user,$host) = split /\@/, $remote->{host};
+            if (!defined $host) {
+                $host = $user;
+                $user = $remote->{user};
+            }
+            my $ssh = Net::SSH::Perl->new($host);
+            if ($remote->{password}) {
+                my $login_ok = $ssh->login($user, $remote->{password}, 1);
+                print TTY "login: $login_ok\n";
+                ...;
+            } else {
+            }
+        } else {
+            croak "Alternate ssh support for remote option not implemented";
+        }
+        deinit_child();
+        exit $? >> 8;
+    } else {
+        croak "Forks::Super: Only 'ssh' protocol supported for remote";
+    }
+}
+
 sub _postlaunch_child_to_sub {
     my $job = shift;
     my $sub = $job->{sub};
@@ -1278,13 +1416,13 @@ sub terminate {
 #
 sub _preconfig {
     my $job = shift;
-
     $job->_preconfig_style;
     $job->_preconfig_dir;
     $job->_preconfig_busy_action;
     $job->_preconfig_start_time;
     $job->_preconfig_dependencies;
     $job->_preconfig_share;
+    $job->_preconfig_remote;
     Forks::Super::Job::Callback::_preconfig_callbacks($job);
     Forks::Super::Job::OS::_preconfig_os($job);
     return;
@@ -1500,6 +1638,31 @@ sub _preconfig_dependencies {
     return;
 }
 
+sub _preconfig_remote {
+    my $job = shift;
+    if (defined $job->{remote}) {
+        if (ref($job->{remote}) ne 'HASH') {
+            $job->{remote} = { host => $job->{remote} };
+        }
+        $job->{remote}{proto} ||= 'ssh';
+        $job->{remote}{__hosts__} = $job->{remote}{host};
+
+        if (!$job->{cmd}) {
+            if ($job->{exec}) {
+                $job->{cmd} = $job->{exec};
+            } else {
+                carp "fork: remote => ... specified without  cmd => ... ! ",
+                     "remote spec will be ignored";
+                $job->{remote_disabled} = delete $job->{remote};
+            }
+        }
+    }
+    if (defined $job->{rhost}) {
+        croak "Use  remote => ...  not  rhost => ...";
+    }
+    return;
+}
+
 # convert job names in an array to job ids, if necessary
 sub _resolve_names {
     my ($job, $id_list) = @_;
@@ -1931,6 +2094,16 @@ sub count_processes {
     return @n;
 }
 
+sub count_active_processes_on_host {
+    my $host = shift;
+    return scalar grep { 
+        $_->{remote} && $_->{remote}{host} &&
+            $_->{remote}{host} eq $host &&
+            defined($_->{state})
+            && $_->{state} eq 'ACTIVE'
+    } @ALL_JOBS;
+}
+
 sub init_child {
     Forks::Super::Job::Ipc::init_child();
     return;
@@ -2034,7 +2207,7 @@ Forks::Super::Job - object representing a background task
 
 =head1 VERSION
 
-0.74
+0.75
 
 =head1 SYNOPSIS
 
