@@ -33,7 +33,7 @@ $| = 1;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(close_fh);
-our $VERSION = '0.74';
+our $VERSION = '0.75';
 
 our (%FILENO, %SIG_OLD, $IPC_COUNT, $IPC_DIR_DEDICATED,
      @IPC_FILES, %IPC_FILES);
@@ -150,7 +150,15 @@ sub _safeopen ($*$$;%) {
     }
 
     if ($__OPEN_FH > 0.98 * $__MAX_OPEN_FH) {
-	carp sprintf "This program now using %d of ~%d open file handles. Consider closing unused file handles.", $__OPEN_FH, $__MAX_OPEN_FH;
+        if ($job->{debug}) {
+            debug("This program now using $__OPEN_FH of ~$__MAX_OPEN_FH " .
+                  "open file handles. Consider closing unused file " .
+                  "handles");
+        } else {
+            carp("This program now using $__OPEN_FH of ~$__MAX_OPEN_FH " .
+                  "open file handles. Consider closing unused file " .
+                  "handles");
+        }
 	$Forks::Super::Job::Ipc::TOO_MANY_OPEN_FH_WARNINGS++;
 
 	try_to_close_some_open_filehandles();
@@ -355,9 +363,6 @@ sub try_to_close_some_open_filehandles {
 	}
     }
 
-
-#   print STDERR "--------------\n";
-
     # close jobs that are
     #    * old
     #    * done (have a status)
@@ -365,6 +370,8 @@ sub try_to_close_some_open_filehandles {
     if ($nclosed) {
 	warn "Forks::Super::Job::Ipc: Closed $nclosed open file handles from jobs that look complete.\n";
 	$__OPEN_FH -= $nclosed;
+    } else {
+        warn "Forks::Super::Job::Ipc: unable to close any open filehandles";
     }
 
     return;
@@ -387,23 +394,23 @@ sub __set_fh_config_from_spec {
 	$config->{all} = 1;
 	$config->{stress} = 1 if $Forks::Super::Config::IS_TEST;
     } else {
-	if ($fh_spec =~ /(?<!jo)in/i) {
+	if ($fh_spec =~ /\bin\b/i) {
 	    $config->{in} = 1;
 	}
-	if ($fh_spec =~ /out/i) {
+	if ($fh_spec =~ /\bout\b/i) {
 	    $config->{out} = 1;
 	}
-	if ($fh_spec =~ /err/i) {
+	if ($fh_spec =~ /\berr\b/i) {
 	    $config->{err} = 1;
 	}
-	if ($fh_spec =~ /join/i) {
+	if ($fh_spec =~ /\bjoin\b/i) {
 	    $config->{join} = 1;
 	    $config->{out} = 1;
 	    $config->{err} = 1;
 	}
     }
 
-    if ($fh_spec =~ /block/i) {
+    if ($fh_spec =~ /\bblock\b/i) {
 	$config->{block} = 1;
     }
 
@@ -1028,7 +1035,7 @@ for interprocess communication.
 It should be/have been cleaned up when the process completes/completed.
 If that didn't happen for some reason, it is safe to delete
 this directory. You may also consider running the command
-"perl -MForks::Super=cleanse,$_IPC_DIR" to clean up this
+"$^X -MForks::Super=cleanse,$_IPC_DIR" to clean up this
 and any other IPC litter.
 
 ____
