@@ -18,7 +18,7 @@ use warnings;
 use constant FOREVER => 9E9;
 use constant LONG_TIME => 9E8;
 
-our $VERSION = '0.74';
+our $VERSION = '0.75';
 
 our $MAIN_PID = $$;
 our $DISABLE_INT = 0;
@@ -174,7 +174,11 @@ sub _child_timeout {
 	return;
     }
 
-    warn "Forks::Super: child process timeout\n";
+    if ($DEBUG) {
+        debug "Forks::Super: child process timeout\n";
+    } else {
+        warn "Forks::Super: child process timeout\n";
+    }
     $TIMEDOUT = 1;
 
     # we wish to kill not only this child process,
@@ -192,6 +196,9 @@ sub _child_timeout {
 	    push @to_kill, $Forks::Super::Job::CHILD_EXEC_PID;
 	}
 	if (@to_kill > 0) {
+            if ($DEBUG) {
+                debug("to kill[$TIMEOUT_SIG]: @to_kill");
+            }
 	    Forks::Super::kill($TIMEOUT_SIG, @to_kill);
 	}
     } elsif (_child_timeout_has_new_process_group()) {
@@ -208,6 +215,9 @@ sub _child_timeout {
     } elsif (&IS_WIN32) {
 	_child_timeout_Win32();
     }
+    if ($DEBUG) {
+        debug("_child_timeout: end reached");
+    }
     exit 255;
 }
 
