@@ -86,7 +86,7 @@ my $really_quiet = 0;
 my $abort_on_first_error = 0;
 my $use_harness = '';
 my $shuffle = '';
-my $debug = '';
+my $debug = $ENV{DEBUG} || '';
 my $use_socket = '';
 my $help = '';
 my $pause = 0;
@@ -754,7 +754,7 @@ sub maxproc_initial {
     if (@mask < $n) {
 	$n = @mask || $n;
     }
-    return int(2 * $n + 1);
+    return $n < 10 ? int(2 * $n + 1) : $n + 6;
 }
 
 # if appropriate and suppported, enhance output to STDOUT with color.
@@ -819,7 +819,7 @@ forked_harness.pl - run tests in parallel with Forks::Super
 
 =head1 VERSION
 
-0.74
+0.75
 
 =head1 SYNOPSIS
 
@@ -1103,7 +1103,7 @@ for each test that it runs:
 
 The test results are reported when a test finishes. The reports may
 be out of order. If there is more than one iteration of testing planned
-(see the C<-r> option), this this also reports the current iteration.
+(see the C<-r> option), this also reports the current iteration.
 
 =item C<B> - total number of tests
 
@@ -1124,7 +1124,7 @@ The highest exit code from any previous test.
 
 =item C<E> - test time
 
-Running time of the test, with microsecond resolution. When you get
+Running time of the test, with millisecond resolution. When you get
 in the habit of running your unit tests in parallel, the longest 
 running tests in your suite can be a bottleneck in your overall test time,
 and you may want to look for ways to break it into several smaller
